package helloworlddsl;

import org.codehaus.groovy.ast.ASTNode;
import org.codehaus.groovy.ast.MethodNode;
import org.codehaus.groovy.ast.expr.ArgumentListExpression;
import org.codehaus.groovy.ast.expr.ConstantExpression;
import org.codehaus.groovy.ast.expr.DeclarationExpression;
import org.codehaus.groovy.ast.expr.Expression;
import org.codehaus.groovy.ast.expr.MethodCallExpression;
import org.codehaus.groovy.ast.stmt.BlockStatement;
import org.codehaus.groovy.ast.stmt.ExpressionStatement;
import org.codehaus.groovy.ast.stmt.Statement;
import org.codehaus.groovy.control.CompilePhase;
import org.codehaus.groovy.control.SourceUnit;
import org.codehaus.groovy.transform.ASTTransformation;
import org.codehaus.groovy.transform.GroovyASTTransformation;

@GroovyASTTransformation(phase=CompilePhase.SEMANTIC_ANALYSIS)
public class MyGroovyASTTransformer implements ASTTransformation {
	public void visit(ASTNode[] nodes, SourceUnit sourceUnit) {
		for (ASTNode node : nodes) {
			System.out.println("Node == " + node.getClass().getCanonicalName());
			if (node instanceof MethodNode) {
				MethodNode methodNode = (MethodNode)node;
//				for (Statement stmt : sourceUnit.getAST().getStatementBlock().getStatements()) {
//					crawlStatement(stmt);
//					System.out.println("----------------------------------------");
//				}
				crawlStatement(((MethodNode) node).getCode());
			}
		}
    }
	
	private void crawlStatement(Statement stmt) {
		System.out.println("----------------------------------------");
		System.out.println("Stmt == " + stmt);
		System.out.println("Stmt code == " + stmt.getText());
		if (stmt instanceof ExpressionStatement) {
			ExpressionStatement expressionStmt = (ExpressionStatement)stmt;
			crawlExpression("Expression under statement", expressionStmt.getExpression());
		} else if (stmt instanceof BlockStatement) {
			BlockStatement blockStmt = (BlockStatement)stmt;
			for (Statement substmt : blockStmt.getStatements()) {
				crawlStatement(substmt);
			}
		}
	}
	
	private void crawlExpression(String tag, Expression expression) {
		System.out.println("----" + tag + "----");
		if (expression.toString() != null && expression.toString().contains("addRule")) {
			System.out.println("addRule expression =====" + expression);
		} else {
			System.out.println("Expression == " + expression);
		}
		if (expression instanceof MethodCallExpression) {
			MethodCallExpression mcallexp = (MethodCallExpression)expression;
			System.out.println("Method call expression calling == " + mcallexp.getMethodAsString());
			crawlExpression("Methods", mcallexp.getMethod());
			crawlExpression("Args", mcallexp.getArguments());
		} else if (expression instanceof ConstantExpression) {
			ConstantExpression cexp = (ConstantExpression)expression;
			System.out.println("const exp name: " + cexp.getConstantName());
		} else if (expression instanceof ArgumentListExpression) {
			ArgumentListExpression argListExpr = (ArgumentListExpression)expression;
			for (Expression argExpr : argListExpr.getExpressions()) {
				crawlExpression("Arg", argExpr);
			}
		} else if (expression instanceof DeclarationExpression) {
			DeclarationExpression declExpr = (DeclarationExpression)expression;
			crawlExpression("Right of decl expr", declExpr.getRightExpression());
		}
	}
}
